# This code is copyrighted by Stephan Richter in July, 2001.
#
# It is Open Source. See the LICENSE.txt for more details!
#
# written by Stephan Richter
# Last updated: July 28, 2001
import Globals, os, string
from OFS.Folder     import Folder
from ExternalObject import ExternalObject

# import the necessary External File Types
from ExternalDTMLMethod   import ExternalDTMLMethod
from ExternalPythonScript import ExternalPythonScript
from ExternalImage        import ExternalImage
from ExternalFile         import ExternalFile


# A list of the file types that are recognized by the ProductView
DTMLMethodTypes = ('dtml', 'html', 'txt')
PyScriptTypes   = ('py',)
ImageTypes      = ('gif', 'jpg', 'png')
FileTypes       = ('dat', 'zexp', 'xml')
ExcludedTypes   = ('pyc',)


manage_addExternalFolderForm = Globals.DTMLFile('dtml/manage_addExternalFolderForm', globals())

def manage_addExternalFolder(self, id, REQUEST=None):
    '''manage_addExternalFolder(self, id, REQUEST=None) --> add an External Folder; return HTML'''

    # create the new directory (folder)
    os.mkdir(os.path.join(self._getPath(), id))

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class ExternalFolder(ExternalObject, Folder):
    '''ExternalFolder class - Represent a folder in the file system'''

    meta_type = 'External Folder'
    
    # let's minimize the options, since the External Folder is not a persistent obeject
    manage_options = (Folder.manage_options[0],)
    _meta_types = ()


    def __init__(self, id, path):
        '''__init__(self, id, path) --> Initialize the object; return None'''
        self.id = str(id)
        self.path = path
        

    def __bobo_traverse__(self, REQUEST, name):
        '''__bobo_traverse__(self, REQUEST, name) --> define how objects can be
           found via the URL; return object'''
        # First try to find the object in the file system
        try:
            return self._getOb(name)
        except:
            pass
        # Maybe it is an attribute? 
        try:
            return getattr(self, name)
        except AttributeError:
            pass
        # Nothing found; Error.
        REQUEST.RESPONSE.notFoundError(name)


    def _getPath(self):
        '''_getPath(self) --> Get the external path of the folder; return string'''
        return os.path.join(Globals.SOFTWARE_HOME, 'Products', self.path)


    def all_meta_types(self):
        '''all_meta_types(self) --> Overload this method to implement only certain objects
           that can be added; in our case the External Object Series; return list'''
        return self._meta_types


    def manage_renameObject(self, id, new_id, REQUEST=None):
        '''manage_renameObject(self, id, new_id, REQUEST=None) --> reimplement renaming
           return None'''
        os.rename(os.path.join(self._getPath(), id),
                  os.path.join(self._getPath(), new_id)) 


    def objectIds(self, spec=None):
        '''objectIds(self, spec=None) --> Find the Ids of the objects that are contained
           by the external folder; return list'''
        # get all files and directories inside this external directory
        fileNames = os.listdir(self._getPath())
        accepted = []
        for fileName in fileNames:
            if os.path.isdir(os.path.join(self._getPath(), fileName)):
                accepted.append(fileName)
            elif string.split(fileName, '.')[-1] not in ExcludedTypes:
                accepted.append(fileName)
        return accepted
    

    def _delOb(self, id):
        '''_delOb(self, id) --> delete an object from the external files; retrun None'''
        path = os.path.join(self._getPath(), id)
        if os.path.isdir(path):
            os.rmdir(path)
        else:
            os.remove(path)
                  

    def _getOb(self, id, default=[]):
        '''_getOb(self, id, default=[]) --> get an object based on its id'''

        # generate the file path
        filePath = os.path.join(self._getPath(), id)

        # if the id is pointing to a directory, then create a folder and return it
        if os.path.isdir(filePath):
            return ExternalFolder(id, os.path.join(self.path, id)).__of__(self)

        elif os.path.isfile(filePath):
            # find the extension of the file
            ext = string.split(id, '.')[-1]
            # Open the data file, so we can read the data
            file = open(filePath)
            
            if ext in PyScriptTypes:
                object = ExternalPythonScript(id)
                object.write(file.read())

            elif ext in DTMLMethodTypes:
                object = ExternalDTMLMethod(file.read(), __name__=id)

            elif ext in ImageTypes:
                object = ExternalImage(id, '', '', '')
                object.manage_upload(file)

            else:
                object = File(id, '', '', '')
                object.manage_upload(file)

            # close the data file
            file.close()

            # Now return the object as a child of self
            return object.__of__(self)

        # if it was not possible to find the object, raise an acception
        raise 'NotFound', 'The object with the id "%s" was not found in the file system' %id
